@@ -1,8 +1,12 @@
+from __future__ import absolute_import
+
 import re
+
 import attr
 
 from kslurm.args import ShapeArg, TailArg
 
+# v(major_version).(minor_version).(patch_version).(sub_patch_version)
 VERSION_REGEX = (
     r"v?(\d+)(?:\.(\d+))?(?:\.(\d+))?(?:\.(\d+))?"
     "("
@@ -13,17 +17,16 @@ VERSION_REGEX = (
     r"(?:\+[^\s]+)?"
 )
 
+
 @attr.s(auto_attribs=True)
 class UpdateModel:
     version: ShapeArg[str] = ShapeArg[str](
-        match = lambda arg: bool(re.match(VERSION_REGEX, arg)),
-        value = "",
+        match=lambda arg: bool(re.match(VERSION_REGEX, arg)),
+        value="",
         name="Version",
         syntax="d[.d][.d][.d][_tag]",
-        examples=[
-            "0.1.0"
-        ],
-        help="Optional: version to install"
+        examples=["0.1.0"],
+        help="Optional: version to install",
     )
 
-    tail: TailArg = TailArg()
\ No newline at end of file
+    tail: TailArg = TailArg()
